/****************************************************************************************/
/* Racer1.c - functions and data for racer #1                                           */
/*--------------------------------------------------------------------------------------*/

#define  P1_START_X         42
#define  P1_START_Y        200

int  p1x[TRAIL_LENGTH];
int  p1y[TRAIL_LENGTH];

int  p1QueueOldest;
int  p1QueueNewest;
int  p1CurDirection;
int  p1NewDirection;

int  p1Speed;

/*......................................................................................*/
/* player 1 temp variables.                                                             */
/*......................................................................................*/

int   p1Index;
int   p1NextNewest;
int   p1NextOldest;
int   p1CheckX;
int   p1CheckY;
int   p1HitColor;

/*--------------------------------------------------------------------------------------*/
/* initP1() - initialize player # 1.                                                    */
/*--------------------------------------------------------------------------------------*/

initP1()
{
	/*........................................................................*/
	/* initialize screen location.                                            */
	/*........................................................................*/  

	p1x[0] = p1x[1] = P1_START_X;
	p1y[0] = p1y[1] = P1_START_Y;
	p1CurDirection  = MOVE_UP;
    p1Speed         = SP_MEDIUM;
	
	/*........................................................................*/
	/* initialize queue pointers.                                             */
	/*........................................................................*/
	
	p1QueueOldest = 0;
	p1QueueNewest = 1;

	/*........................................................................*/
	/* initialize sprite location.                                            */
	/*........................................................................*/

	spr_set( 1 );
	spr_pal(RacerPalette);
	spr_x( p1x[p1QueueNewest] );
	spr_y( p1y[p1QueueNewest] );
  
}
/*--------------------------------------------------------------------------------------*/
/* p1UpdateSprite() - update player 1 sprite location.                                  */
/*--------------------------------------------------------------------------------------*/

p1UpdateSprite()
{
	spr_set( 1 );
	
	/*........................................................................*/
	/* Directions.                                                            */
	/*........................................................................*/

	if( p1CurDirection == MOVE_UP    )  
	{
		spr_ctrl( FLIP_MAS, 0);
		spr_pattern( 0x0C00 + bike[0] * 128);
		spr_x( p1x[p1QueueNewest] - 8);
		spr_y( p1y[p1QueueNewest] - 4);
	}
	if( p1CurDirection == MOVE_DOWN  )
	{
		spr_ctrl( FLIP_MAS, FLIP_Y );
		spr_pattern( 0x0C00 + bike[0] * 128 );
		spr_x( p1x[p1QueueNewest] - 8);
		spr_y( p1y[p1QueueNewest] - 4);
	}

	if( p1CurDirection == MOVE_RIGHT )
	{
		spr_ctrl( FLIP_MAS, 0 );
		spr_pattern( (0x0C00 + bike[0] * 128) + 0x40);
		spr_x( p1x[p1QueueNewest] - 4);
		spr_y( p1y[p1QueueNewest] - 8);
	}
	if( p1CurDirection == MOVE_LEFT  )
	{
		spr_ctrl( FLIP_MAS, FLIP_X );
		spr_pattern( (0x0C00 + bike[0] * 128) + 0x40 );
		spr_x( p1x[p1QueueNewest] - 4);
		spr_y( p1y[p1QueueNewest] - 8);
	}
}

/*--------------------------------------------------------------------------------------*/
/* p1CheckCollision() - check for player 1 collision.                                   */
/*--------------------------------------------------------------------------------------*/

p1CheckCollision()
{
    p1CheckX = p1x[ p1QueueNewest ] + DeltaX[ p1CurDirection ];
	p1CheckY = p1y[ p1QueueNewest ] + DeltaY[ p1CurDirection ];
	
	for( p1Index = 1; p1Index <= p1Speed; p1Index ++ )
	{
		p1CheckX = p1CheckX + DeltaX[ p1CurDirection ];
		p1CheckY = p1CheckY + DeltaY[ p1CurDirection ];
		
		if( gfx_point( p1CheckX, p1CheckY ) != 0 ) 
		{
		   return( 1 );
		}
    }
	return( 0 );
}
/*--------------------------------------------------------------------------------------*/
/* p1UpdateTrail() - update player ones trail; this updates the endpont que as well.    */
/*--------------------------------------------------------------------------------------*/

p1UpdateTrail()
{
	p1NewDirection = p1GetDirection();
	if( p1NewDirection != p1CurDirection )
	{
        /*..........................................................................*/
		/* if line array is full, erase oldest line.                                */
		/*..........................................................................*/
		
        if( p1QueueFull() )
		{
		    p1NextOldest = p1QNextOldest();
			p1EraseTrail();
		    p1QueueOldest = p1NextOldest;
		}
		
        /*..........................................................................*/
        /* add new line to line array.                                              */
        /*..........................................................................*/

		p1NextNewest = p1QNextNewest( p1QueueNewest );

		p1x[p1NextNewest] = p1x[p1QueueNewest];
		p1y[p1NextNewest] = p1y[p1QueueNewest];
		p1QueueNewest     = p1NextNewest;
		p1CurDirection    = p1NewDirection;
    }
	

}
/*--------------------------------------------------------------------------------------*/
/* p1DrawTrail() - draw trail for player 1.                                             */
/*--------------------------------------------------------------------------------------*/

p1DrawTrail()
{
	if( p1CurDirection == MOVE_UP    )  
	{
		gfx_line( p1x[p1QueueNewest], 
				  p1y[p1QueueNewest],
				  p1x[p1QueueNewest] + DeltaX[ p1CurDirection] * p1Speed, 
				  p1y[p1QueueNewest] + DeltaY[ p1CurDirection] * p1Speed, 8 );
				  
		gfx_line( p1x[p1QueueNewest] - 1, 
				  p1y[p1QueueNewest],
				  p1x[p1QueueNewest] - 1 + DeltaX[ p1CurDirection] * p1Speed, 
				  p1y[p1QueueNewest]     + DeltaY[ p1CurDirection] * p1Speed, 8 );
    }
	if( p1CurDirection == MOVE_DOWN    )  
	{
		gfx_line( p1x[p1QueueNewest], 
				  p1y[p1QueueNewest],
				  p1x[p1QueueNewest] + DeltaX[ p1CurDirection] * p1Speed, 
				  p1y[p1QueueNewest] + DeltaY[ p1CurDirection] * p1Speed, 8 );
				  
		gfx_line( p1x[p1QueueNewest] - 1, 
				  p1y[p1QueueNewest],
				  p1x[p1QueueNewest] - 1 + DeltaX[ p1CurDirection] * p1Speed, 
				  p1y[p1QueueNewest]     + DeltaY[ p1CurDirection] * p1Speed, 8 );
    }

	if( p1CurDirection == MOVE_LEFT    )  
	{
		gfx_line( p1x[p1QueueNewest], 
				  p1y[p1QueueNewest],
				  p1x[p1QueueNewest] + DeltaX[ p1CurDirection] * p1Speed, 
				  p1y[p1QueueNewest] + DeltaY[ p1CurDirection] * p1Speed, 8 );
				  
		gfx_line( p1x[p1QueueNewest],
				  p1y[p1QueueNewest] -1,
				  p1x[p1QueueNewest]    + DeltaX[ p1CurDirection] * p1Speed, 
				  p1y[p1QueueNewest] -1 + DeltaY[ p1CurDirection] * p1Speed, 8 );
    }
	if( p1CurDirection == MOVE_RIGHT    )  
	{
		gfx_line( p1x[p1QueueNewest], 
				  p1y[p1QueueNewest],
				  p1x[p1QueueNewest] + DeltaX[ p1CurDirection] * p1Speed, 
				  p1y[p1QueueNewest] + DeltaY[ p1CurDirection] * p1Speed, 8 );
				  
		gfx_line( p1x[p1QueueNewest], 
				  p1y[p1QueueNewest] - 1,
				  p1x[p1QueueNewest]     + DeltaX[ p1CurDirection] * p1Speed, 
				  p1y[p1QueueNewest] - 1 + DeltaY[ p1CurDirection] * p1Speed, 8 );
    }

}
/*--------------------------------------------------------------------------------------*/
/* p1EraseTrail() - erase first trail in queue.                                         */
/* note: this relys on ous not having turned yet.                                       */
/*--------------------------------------------------------------------------------------*/

p1EraseTrail()
{
    p1NextOldest = p1QNextOldest();
		
	if( p1x[p1QueueOldest] == p1x[p1NextOldest]    )
	{
		gfx_line( p1x[p1QueueOldest], p1y[p1QueueOldest],
				  p1x[p1NextOldest ], p1y[p1NextOldest ], 0 );
				  
		gfx_line( p1x[p1QueueOldest] - 1, p1y[p1QueueOldest],
				  p1x[p1NextOldest ] - 1, p1y[p1NextOldest ], 0 );
    }
	if( p1y[p1QueueOldest] == p1y[p1NextOldest]    )
	{
		gfx_line( p1x[p1QueueOldest], p1y[p1QueueOldest],
				  p1x[p1NextOldest ], p1y[p1NextOldest ], 0 );
				  
		gfx_line( p1x[p1QueueOldest], p1y[p1QueueOldest] - 1,
				  p1x[p1NextOldest ], p1y[p1NextOldest ] - 1, 0 );
    }
}

/*--------------------------------------------------------------------------------------*/
/* p1UpdateDeath() - update player 1 dead sprite location.                              */
/*--------------------------------------------------------------------------------------*/

p1UpdateDeath()
{
	spr_set( 1 );
	
	/*........................................................................*/
	/* Directions.                                                            */
	/*........................................................................*/
	if(frame % 4 == 0)	
	{
		spr_pattern( ExplodeAddress );
	}
	else
	{
		
		if(frame % 3 == 0) 
		{
			spr_ctrl(FLIP_MAS, FLIP_X);
			spr_pattern(ExplodeAddress + 0x40);
		}
		else if(frame % 2 == 0)
		{
			spr_ctrl(FLIP_MAS, FLIP_Y);
			spr_pattern(ExplodeAddress + 0x40);
		}
		else
		{
			spr_ctrl(FLIP_MAS, 0);
			spr_pattern(ExplodeAddress + 0x40);
		}	
	}
	
	
	if( p1CurDirection == MOVE_UP    )  
	{		
		spr_x( p1x[p1QueueNewest] - 8);
		spr_y( p1y[p1QueueNewest] - 4);
	}
	if( p1CurDirection == MOVE_DOWN  )
	{
		spr_x( p1x[p1QueueNewest] - 8);
		spr_y( p1y[p1QueueNewest] - 4);
	}

	if( p1CurDirection == MOVE_RIGHT )
	{
		spr_x( p1x[p1QueueNewest] - 4);
		spr_y( p1y[p1QueueNewest] - 8);
	}
	if( p1CurDirection == MOVE_LEFT  )
	{
		spr_x( p1x[p1QueueNewest] - 4);
		spr_y( p1y[p1QueueNewest] - 8);
	}
}
