/****************************************************************************************/
/* Racer2.c - functions and data for racer #2                                           */
/*--------------------------------------------------------------------------------------*/

#define  P2_START_X        128
#define  P2_START_Y        200

int  p2x[TRAIL_LENGTH];
int  p2y[TRAIL_LENGTH];

int  p2QueueOldest;
int  p2QueueNewest;
int  p2CurDirection;
int  p2NewDirection;

int  p2Speed;

/*......................................................................................*/
/* player 1 temp variables.                                                             */
/*......................................................................................*/

int   p2Index;
int   p2NextNewest;
int   p2NextOldest;
int   p2CheckX;
int   p2CheckY;
int   p2HitColor;

/*--------------------------------------------------------------------------------------*/
/* initP2() - initialize player # 2.                                                    */
/*--------------------------------------------------------------------------------------*/

initP2()
{
	/*........................................................................*/
	/* initialize screen location.                                            */
	/*........................................................................*/  

	p2x[0] = p2x[1] = P2_START_X;
	p2y[0] = p2y[1] = P2_START_Y;
	
	p2CurDirection  = MOVE_UP;
    p2Speed         = SP_MEDIUM;
	
	/*........................................................................*/
	/* initialize queue pointers.                                             */
	/*........................................................................*/
	
	p2QueueOldest = 0;
	p2QueueNewest = 1;

	/*........................................................................*/
	/* initialize sprite location.                                            */
	/*........................................................................*/

	spr_set( 2 );
	spr_x( p2x[p2QueueNewest] );
	spr_y( p2y[p2QueueNewest] );
  
}
/*--------------------------------------------------------------------------------------*/
/* p2UpdateSprite() - update player 2 sprite location.                                  */
/*--------------------------------------------------------------------------------------*/

p2UpdateSprite()
{
	spr_set( 2 );
	
	/*........................................................................*/
	/* Directions.                                                            */
	/*........................................................................*/

	if( p2CurDirection == MOVE_UP    )  
	{
		spr_ctrl( FLIP_MAS, 0);
		spr_pattern( 0x0C00 + bike[1] * 128 );
		spr_x( p2x[p2QueueNewest] - 8);
		spr_y( p2y[p2QueueNewest] - 4);
	}
	if( p2CurDirection == MOVE_DOWN  )
	{
		spr_ctrl( FLIP_MAS, FLIP_Y );
		spr_pattern( 0x0C00 + bike[1] * 128 );
		spr_x( p2x[p2QueueNewest] - 8);
		spr_y( p2y[p2QueueNewest] - 4);
	}
	if( p2CurDirection == MOVE_RIGHT )
	{
		spr_ctrl( FLIP_MAS, 0 );
		spr_pattern( 0x0C00 + bike[1] * 128 + 0x40);
		spr_x( p2x[p2QueueNewest] - 4);
		spr_y( p2y[p2QueueNewest] - 8);
	}
	if( p2CurDirection == MOVE_LEFT  )
	{
		spr_ctrl( FLIP_MAS, FLIP_X );
		spr_pattern( 0x0C00 + bike[1] * 128 + 0x40 );
		spr_x( p2x[p2QueueNewest] - 4);
		spr_y( p2y[p2QueueNewest] - 8);
	}
}

/*--------------------------------------------------------------------------------------*/
/* p2CheckCollision() - check for player 2 collision.                                   */
/*--------------------------------------------------------------------------------------*/

p2CheckCollision()
{
    p2CheckX = p2x[ p2QueueNewest ] + DeltaX[ p2CurDirection ];
	p2CheckY = p2y[ p2QueueNewest ] + DeltaY[ p2CurDirection ];
	
	for( p2Index = 1; p2Index <= p2Speed; p2Index ++ )
	{
		p2CheckX = p2CheckX + DeltaX[ p2CurDirection ];
		p2CheckY = p2CheckY + DeltaY[ p2CurDirection ];
		
		if( gfx_point( p2CheckX, p2CheckY ) != 0 ) 
		{
		   return( 1 );
		}
    }
	return( 0 );
}
/*--------------------------------------------------------------------------------------*/
/* p2UpdateTrail() - update player twos trail; this updates the endpont que as well.    */
/*--------------------------------------------------------------------------------------*/

p2UpdateTrail()
{
	p2NewDirection = p2GetDirection();
	if( p2NewDirection != p2CurDirection )
	{
        /*..........................................................................*/
		/* if line array is full, erase oldest line.                                */
		/*..........................................................................*/
		
        if( p2QueueFull() )
		{
		    p2NextOldest = p2QNextOldest();
			p2EraseTrail();
		    p2QueueOldest = p2NextOldest;
		}
		
        /*..........................................................................*/
        /* add new line to line array.                                              */
        /*..........................................................................*/

		p2NextNewest = p2QNextNewest( p2QueueNewest );

		p2x[p2NextNewest] = p2x[p2QueueNewest];
		p2y[p2NextNewest] = p2y[p2QueueNewest];
		p2QueueNewest     = p2NextNewest;
		p2CurDirection    = p2NewDirection;
    }
}
/*--------------------------------------------------------------------------------------*/
/* p2DrawTrail() - draw trail for player 2.                                             */
/*--------------------------------------------------------------------------------------*/

p2DrawTrail()
{
	if( p2CurDirection == MOVE_UP    )  
	{
		gfx_line( p2x[p2QueueNewest], 
				  p2y[p2QueueNewest],
				  p2x[p2QueueNewest] + DeltaX[ p2CurDirection] * p2Speed, 
				  p2y[p2QueueNewest] + DeltaY[ p2CurDirection] * p2Speed, 9 );
				  
		gfx_line( p2x[p2QueueNewest] - 1, 
				  p2y[p2QueueNewest],
				  p2x[p2QueueNewest] - 1 + DeltaX[ p2CurDirection] * p2Speed, 
				  p2y[p2QueueNewest]     + DeltaY[ p2CurDirection] * p2Speed, 9 );
    }
	if( p2CurDirection == MOVE_DOWN    )  
	{
		gfx_line( p2x[p2QueueNewest], 
				  p2y[p2QueueNewest],
				  p2x[p2QueueNewest] + DeltaX[ p2CurDirection] * p2Speed, 
				  p2y[p2QueueNewest] + DeltaY[ p2CurDirection] * p2Speed, 9 );
				  
		gfx_line( p2x[p2QueueNewest] - 1, 
				  p2y[p2QueueNewest],
				  p2x[p2QueueNewest] - 1 + DeltaX[ p2CurDirection] * p2Speed, 
				  p2y[p2QueueNewest]     + DeltaY[ p2CurDirection] * p2Speed, 9 );
    }

	if( p2CurDirection == MOVE_LEFT    )  
	{
		gfx_line( p2x[p2QueueNewest], 
				  p2y[p2QueueNewest],
				  p2x[p2QueueNewest] + DeltaX[ p2CurDirection] * p2Speed, 
				  p2y[p2QueueNewest] + DeltaY[ p2CurDirection] * p2Speed, 9 );
				  
		gfx_line( p2x[p2QueueNewest],
				  p2y[p2QueueNewest] -1,
				  p2x[p2QueueNewest]    + DeltaX[ p2CurDirection] * p2Speed, 
				  p2y[p2QueueNewest] -1 + DeltaY[ p2CurDirection] * p2Speed, 9 );
    }
	if( p2CurDirection == MOVE_RIGHT    )  
	{
		gfx_line( p2x[p2QueueNewest], 
				  p2y[p2QueueNewest],
				  p2x[p2QueueNewest] + DeltaX[ p2CurDirection] * p2Speed, 
				  p2y[p2QueueNewest] + DeltaY[ p2CurDirection] * p2Speed, 9 );
				  
		gfx_line( p2x[p2QueueNewest], 
				  p2y[p2QueueNewest] - 1,
				  p2x[p2QueueNewest]     + DeltaX[ p2CurDirection] * p2Speed, 
				  p2y[p2QueueNewest] - 1 + DeltaY[ p2CurDirection] * p2Speed, 9 );
    }

}
/*--------------------------------------------------------------------------------------*/
/* p2EraseTrail() - erase first trail in queue.                                         */
/*--------------------------------------------------------------------------------------*/

p2EraseTrail()
{
    p2NextOldest = p2QNextOldest();
		
    /*............................................................................*/
	/* erase vertical lines.                                                      */
	/*............................................................................*/
	
	if( p2x[p2QueueOldest] == p2x[p2NextOldest]    )
	{
		gfx_line( p2x[p2QueueOldest], p2y[p2QueueOldest],
				  p2x[p2NextOldest ], p2y[p2NextOldest ], 0 );
				  
		gfx_line( p2x[p2QueueOldest] - 1, p2y[p2QueueOldest],
				  p2x[p2NextOldest ] - 1, p2y[p2NextOldest ], 0 );
    }
    /*............................................................................*/
	/* erase horizontal lines.                                                    */
	/*............................................................................*/

	if( p2y[p2QueueOldest] == p2y[p2NextOldest]     )
	{
		gfx_line( p2x[p2QueueOldest], p2y[p2QueueOldest],
				  p2x[p2NextOldest ], p2y[p2NextOldest ], 0 );
				  
		gfx_line( p2x[p2QueueOldest], p2y[p2QueueOldest] - 1,
				  p2x[p2NextOldest ], p2y[p2NextOldest ] - 1, 0 );
    }
}
/*--------------------------------------------------------------------------------------*/
/* p2UpdateDeath() - update player 2 sprite location.                                  */
/*--------------------------------------------------------------------------------------*/

p2UpdateDeath()
{
	spr_set( 2 );
	
	/*........................................................................*/
	/* Directions.                                                            */
	/*........................................................................*/
	if(frame % 4 == 0)	
	{
		spr_pattern( ExplodeAddress );
	}
	else
	{
		
		if(frame % 3 == 0) 
		{
			spr_ctrl(FLIP_MAS, FLIP_X);
			spr_pattern(ExplodeAddress + 0x40);
		}
		else if(frame % 2 == 0)
		{
			spr_ctrl(FLIP_MAS, FLIP_Y);
			spr_pattern(ExplodeAddress + 0x40);
		}
		else
		{
			spr_ctrl(FLIP_MAS, 0);
			spr_pattern(ExplodeAddress + 0x40);
		}	
	}
	
	if( p2CurDirection == MOVE_UP    )  
	{
		spr_x( p2x[p2QueueNewest] - 8);
		spr_y( p2y[p2QueueNewest] - 4);
	}
	if( p2CurDirection == MOVE_DOWN  )
	{
		spr_x( p2x[p2QueueNewest] - 8);
		spr_y( p2y[p2QueueNewest] - 4);
	}
	if( p2CurDirection == MOVE_RIGHT )
	{
		spr_x( p2x[p2QueueNewest] - 4);
		spr_y( p2y[p2QueueNewest] - 8);
	}
	if( p2CurDirection == MOVE_LEFT  )
	{
		spr_x( p2x[p2QueueNewest] - 4);
		spr_y( p2y[p2QueueNewest] - 8);
	}
}
