/***************************************************************************/
/* Board.c - functions for managing Board.                                 */
/*-------------------------------------------------------------------------*/

int index;

setupBoard()
{
    /*..........................................................................*/
	/* set bat screen size.                                                     */
    /*..........................................................................*/

	set_screen_size(SCR_SIZE_32x32);
	
    /*..........................................................................*/
	/* load the palettes for the map.                                           */
	/*..........................................................................*/

	set_bgpal( 0, BoardPalette, 2 );	

    /*..........................................................................*/
	/* font loading is now done by menu program. We still have to set the       */
	/* correct palette.                                                         */
    /*..........................................................................*/

	set_font_addr(0x0800);
	load_font(theFont, 64, 0x0800);
	set_font_pal(1);

#asm
	; This loads the BAT into VRAM. 
	stw   #0,<_di
	stb   #BANK(_BoardMap),<_bl
	stw   #_BoardMap,<_si
	stw   #$0380,<_cx
	jsr   load_vram
#endasm		
	
    /*..........................................................................*/
	/* load the tiles into vram.
    /*..........................................................................*/

	load_vram(BoardTilesAddress, BoardTiles, 1024);	

    /*..........................................................................*/
	/* save the board palette for cycling                                       */
    /*..........................................................................*/

	for(index = 0; index < 16; index++)
	{
		oldColors[index] = get_color(index + 16);
	}
}

/*...............................................................................*/
/* pulseBoard(): does a color cycle of entire array except for index 0, then     */
/*               reloads the palette. Color 0 does not get touched because it is */
/*               not part of the rainbow effect.  It is black. Ignore it         */
/*...............................................................................*/

pulseBoard()
{
#asm
	; --------- save last color ----------
	lda #15				; the array is 16 indexes.  15 is the last one. duhr!
	sta _index			; this is going to be used to loop with.
	asl A				
	tax
	
	lda _oldColors,x	
	sta _tempColor
	inx
	lda _oldColors,x
	sta _tempColor+1
	
	;---------- cycle the array -----------
	lda #14	; we will start at our previous index.  We already got color 15.
	
cycleColors:	
	asl A
	tax
	
	;---------- check the condition --------
	lda _index		
	cmp #1			; we loop (for i = 15; i > 1; i--)
	beq endCycle	
	dec A			; we're good. subtract index and 
	sta _index		; store it for the next loop.
	
	;---------- perform the swap -----------
	lda _oldColors,x	; gets low part of our previous.  We are moving it to the right 1 index
	sta _oldColors+2,x	; sticks it like this: [old] ---> [old+2] [old+1]--->[old+3]  
						;(We're moving one index to the right, so add 2 to the current spot since these are ints?
	lda _oldColors+1,x	; get the high part of our previous
	sta _oldColors+3,x	; stick it in the high part of our current
	txa					; put x back
	lsr a				; shift it back
	dec A				; go down one
	jmp cycleColors		; go loop! the loop will shift the X thing left and store it again.
	
endCycle:
	; ---- put temp color into index 1 to complete cycle ----
	lda #1
	asl A
	tax
	lda _tempColor
	sta _oldColors,x
	lda _tempColor+1
	sta _oldColors+1,x
	
	;----- Cycle is complete, now we have to reload the palette.-----
	stb	#1,<_al						; which palette block we load
	stb #BANK(_oldColors),<_bl		; which bank our palette lives in.
	stw #_oldColors,<_si			; the address of our palette
	stb #1,<_cl						; how many palettes are we loading from there?
	jsr load_palette				; go go go.
#endasm
}