/*.....................................................................................*/
/* Gnop.c - GNOP: Not Old-School Pong program.                                         */
/*.....................................................................................*/
/* Copyright (c) 2011 By Michael E. Ward and Andrew 'Arkhan' Darovich						*/
/*				All Rights Reserved.               														*/
/*.....................................................................................*/

#include "huc.h"
/*......................................................................................*/
/* the (more or less) standard includes.                                                */
/*......................................................................................*/

#include "Font.c"
#include "Lib/JoyPad.c"
#include "Lib/tap.c"

#include "Sound/psglib.c"
#include "Sound/sndDefs.h"
#include "sngInit.c"

/*......................................................................................*/

int winner;

#include "paddle.res"
#include "Ball.res"

#include "paddle.h"
#include "Ball.h"

#include "paddle.c"
#include "paddle2.c"

#include "Ball.c"
#include "Deltas.c"

#include "Board.res"
#include "Board.c"

/*------------------------------------------------------------------------------------*/
/* temporaries for testing.                                                           */
/*------------------------------------------------------------------------------------*/

int	psgMainStatus;
int	psgSubStatus;

/*.........................................................................*/
int frame;
int joyNumber;
int rand_seed;


main()
{
  init_satb();
  
  while( 1 )
  {
	  /*.............................................................................*/
	  /* set up the map. If you want a map bigger than the screen, you need to call  */
	  /* set_screen_size before setting up the map to make it work. Otherwise, the   */
	  /* map will wrap around when it displays.                                      */
	  /*.............................................................................*/
	  
	  setupBoard();
	  
	  /*.............................................................................*/
	  /* set up the hero sprite. It's broken up this way so you could conceivably    */
	  /* have different sprite sheets for the hero (ie, different armor, etc).       */
	  /* then you would only have to call heroLoad() to change the graphics.         */
	  /*.............................................................................*/
	  
	  paddleLoad();
	  
	  p1_Setup();
	  p2_Setup();
	  ball_Setup();
	  
	  p1_Display();
	  p2_Display();
	  ball_Display();
	  
	  paddleReset();
	  Serving = 0;

	  /*.............................................................................*/
	  /* have to do this to update the display.                                      */
	  /*.............................................................................*/
	  
	  satb_update();

	  /*.............................................................................*/
	  /* check for tap, get number of players.                                       */
	  /*.............................................................................*/
      
		psgInit( BOTH_300HZ );
		psgOn(   IRQ_TIMER  ); 	/* use timer for now : vsync works okay, but limits speed */
	  
		sngInit();
		psgDelay( 0 );	
		psgPlay(0);
	  
	  /*.............................................................................*/
	  /* print start message.                                                        */
	  /*.............................................................................*/

	  put_string( "PRESS RUN", 11, 11 );

	  /*.............................................................................*/
	  /* wait for run button.                                                        */
	  /*.............................................................................*/

	  joyNumber = 0;
	  frame     = 0;
	  rand_seed = 12;
	  
	  while( !joyRunPressed( 0 ) )
	  {
		  vsync(0);
		  joyNumber++;
		  rand_seed++;
		  frame++;
		  if(frame == 4)
		  {
			pulseBoard();
			frame = 0;
		  }
         if( joySelectReleased( 0 ) ) reboot();

	  }
	  setupBoard();

	  /*.............................................................................*/
	  /* make sure enough frames have elapsed to have read all sticks.               */
	  /*.............................................................................*/

	  while( joyNumber < 5 )
	  {
		  vsync(0);
		  joyNumber++;
		  frame++;
		  if(frame == 4)
		  {
			pulseBoard();
			frame = 0;
		  }
	  }
	  srand( rand_seed );
	  
	  /*.............................................................................*/
	  /* check for second player.                                                    */
	  /*.............................................................................*/
	  
	  p2_Paddle = 0;
	  if( haveTap() )
	  {
	     for( joyNumber = 1; joyNumber < 5; joyNumber++ )
		 {
		     if( flag[ joyNumber ] == 0)
			 {
			    p2_Paddle = joyNumber;
				joyNumber = 6;
			 }
		 }
	  }
	  /*.............................................................................*/
	  /* main processing loop:                                                       */
	  /*   1) read joystick,                                                         */
	  /*   2) figure out what to do with it,                                         */
	  /*   3) move the hero sprite (if necessary),                                   */
	  /*   4) update the sprite location, and finally                                */
	  /*   5) update the screen.                                                     */
	  /*.............................................................................*/
	  /* The VSync waits for vertical-blank, so things don't run                     */
	  /* as fast as possible; but more importantly, it makes sure we can do screen   */
	  /* updates Without causing graphics glitches (assuming they are done right     */
	  /* away). Game mechanics (Tile checks, hit checks, death checks, etc.) would   */
	  /* be done After satb_update().                                                */
	  /*.............................................................................*/

  	  while( BallState != 5 )
	  {
		  vsync(0);
		  frame++;
		  if(frame == 4)
		  {
			pulseBoard();
			frame = 0;
		  }
		  p1_ProcessKeys();
		  p2_ProcessKeys();
		  
		  p1_Move();
		  p2_Move();
		  ball_Update();
		  
		  p1_Display();
		  p2_Display();
		  
		  ball_Display();
		  satb_update();
		}
	}
}
/***************************************************************************/
/* EndGame() - end game, by putting ball into 'game over' state.           */
/*-------------------------------------------------------------------------*/

EndGame()
{
    BallState = 5;
	
	frame = 0;
	while( 1 )
	{
	    vsync( 0 );
		if( winner == 1 )
		{
		    if( joy( 0 ) != 0 )
			{
			   return;
			}
		}
		if( winner == 2 )
		{
		    if( p2_Paddle != 0 )
			{
				if( joy( p2_Paddle ) != 0 )
				{
				   return;
				}
			}
		}
		frame ++;
		if( frame > 300 )
		{
		   return;
		}
  	    if( (frame % 4) == 3)
		{
			pulseBoard();
		}

	}
}
/*......................................................................................*/
/* reboot() - reboot the system.                                                        */
/*......................................................................................*/

reboot()
{
	#asm
	jmp		_reset
	#endasm
}
