/***************************************************************************/
/* Paddle.c - functions for managing paddles.                              */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 2011 By Michael E. Ward and Andrew 'Arkhan' Darovich		*/
/*				All Rights Reserved.               										*/
/*.........................................................................*/


/***************************************************************************/
/* paddlesLoad() - load paddle resources into memory.                      */
/*-------------------------------------------------------------------------*/

paddleLoad()
{
    load_sprites( SpritesAddress,      PaddleSprite,  4 );
	load_palette( PaddlePaletteNo +16, PaddlePalette, 2 );	
}
/***************************************************************************/
/* paddlesReset() - Reset paddle motions.                                  */
/* Also sets up their only values.
/*-------------------------------------------------------------------------*/

paddleReset()
{
	Paddle_Cur_Fdy   = 192;
	Paddle_Cur_Dy    = 1;
	
	Paddle_Cur_In_Fdx = 192;
	Paddle_Cur_In_Dx  = 1;
	
	Paddle_Cur_Out_Fdx = 192;
	Paddle_Cur_Out_Dx  = 1;
}

/***************************************************************************/
/* p1_SetupSprite() - functions for initializing player 1 paddle sprite.   */
/*-------------------------------------------------------------------------*/

p1_SetupSprite()
{
	spr_set(     Player_1_SpriteNo );
	spr_ctrl(	 SIZE_MAS, SZ_16x32);
	spr_x(       Paddle_1_left );
	spr_y(       Paddle_1_top  );
    spr_pattern( PaddleAddress );
	spr_pal(     PaddlePaletteNo + 16	);          /* uses sprite palette area */
	spr_pri( 1 );
}
/***************************************************************************/
/* p1_SetupScore() - functions for initializing player 1 score sprites.    */
/*-------------------------------------------------------------------------*/

p1_SetupScore()
{
	spr_set(     Player_1_ScoreLow );
	spr_x(    24  );
	spr_y(     0  );
    spr_pattern( SpritesAddress );
	spr_pal( PaddlePaletteNo + 16	);          /* uses sprite palette area */
	spr_pri( 1 );

	spr_set(     Player_1_ScoreHigh );
	spr_x(     8  );
	spr_y(     0  );
    spr_pattern( SpritesAddress );
	spr_pal( PaddlePaletteNo + 16	);          /* uses sprite palette area */
	spr_pri( 1 );
}

/***************************************************************************/
/* p1_Setup() - initialize player 1, including his sprite.                 */
/*-------------------------------------------------------------------------*/

p1_Setup()
{
	Paddle_1_Fx    = 0;
    Paddle_1_X     = 8 + 8;
	
	Paddle_1_Fy    = 0;
	Paddle_1_Y     = 120;
	
    Paddle_1_Fdx     = 0;
	Paddle_1_Dx      = 0;
	
	Paddle_1_Fdy     = 0;
	Paddle_1_Dy      = 0;

	p1_SetupSprite();

	p1_Score = 0;
	p1_SetupScore();
}

/***************************************************************************/
/* p1_Display() - display hero, including his sprite.                      */
/*-------------------------------------------------------------------------*/

p1_Display()
{
	spr_set(     Player_1_SpriteNo );
	spr_x(       Paddle_1_left );
	spr_y(       Paddle_1_top  );
	spr_show(    Player_1_SpriteNo     );
}
/***************************************************************************/
/* p1_DisplayScore() - display hero, including his sprite.                      */
/*-------------------------------------------------------------------------*/

p1_DisplayScore()
{
	spr_show(    Player_1_ScoreLow   );
	spr_show(    Player_1_ScoreHigh  );
}
/***************************************************************************/
/* p1_ProcessKeys() - figure out what to do with controller inputs.        */
/*-------------------------------------------------------------------------*/

p1_ProcessKeys()
{
    Paddle_1_Fdx = 0;
	Paddle_1_Dx  = 0;
	
    Paddle_1_Fdy = 0;
    Paddle_1_Dy  = 0;
	
	if( joyUpDown(0)   )  p1_Set_Up_Dy();
	if( joyDownDown(0) )  p1_Set_Down_Dy();
	
	if( joyRightDown(0) ) p1_To_Net();
	if( joyLeftDown(0)  ) p1_From_Net();
}

/***************************************************************************/
/* p1_Move() - update paddle information.                                  */
/*-------------------------------------------------------------------------*/

p1_Move()
{
	/*.............................................................*/
	/* update paddle position.                                     */
	/*.............................................................*/
	
    p1_Update_X();
	p1_Update_Y();
	
	/*.............................................................*/
	/* lock paddle into playable area.                             */
	/*.............................................................*/
	
	if( Paddle_1_top < 24 )
	{
	   Paddle_1_Y   = 24;
	   Paddle_1_Dy  = 0;
	}
	if( Paddle_1_bot >= 216 )
	{
	   Paddle_1_Y   = 184;
	   Paddle_1_Dy  = 0;
	}
	if( Paddle_1_left < 8 )
	{
	   Paddle_1_X = 8;
	}
	if( Paddle_1_right > 119 )
	{
	   Paddle_1_X = 119 - 8;
	}
}
/***************************************************************************/
/* p1_AddPoint() - Give player a point.		                               */
/*-------------------------------------------------------------------------*/

p1_AddPoint()
{
    int low;
	int high;
	
	/*............................................................*/
	/* you only get points if you are serving.                    */
	/*............................................................*/
	
    if( Serving == 1 )
	{
		p1_Score = p1_Score + 1;
	}

	/*............................................................*/
	/* get digits for sprites.                                    */
	/*............................................................*/
	
	high     = p1_Score / 10;
	low      = p1_Score % 10;

	/*............................................................*/
	/* update sprites                                             */
	/*............................................................*/

	spr_set(     Player_1_ScoreLow );
    spr_pattern( SpritesAddress + low * 64 );
	
	spr_set(     Player_1_ScoreHigh );
    spr_pattern( SpritesAddress + high * 64 );

	/*............................................................*/
	/* reset paddle motions.                                      */
	/*............................................................*/
	
	paddleReset();

	/*............................................................*/
	/* check for game over.                                       */
	/*............................................................*/
	
	if( p1_Score >= 11 )
	{
	   /*.....................................................*/
	   /* you have to win by 2.                               */
	   /*.....................................................*/
	   
	   if( p1_Score >= (p2_Score + 2) )
	   {
	        winner = 1;
			put_string( "PLAYER 1 WINS!", 8, 0 );
			EndGame();
			return;
	   }
	   /*.....................................................*/
	   /* and 21-21 is a tie.                                 */ 
	   /*.....................................................*/
	   
	   if( p1_Score >= 21 )
	   {
	       if( p2_Score >= 21 )
		   {
			   winner = 0;
			   put_string( "   TIE GAME   ", 8, 0 );
			   EndGame();
			   return;
		   }
	   }
	}
}


/***************************************************************************/
/* p2_SetupSprite() - functions for initializing player 2 paddle sprite.   */
/*-------------------------------------------------------------------------*/

p2_SetupSprite()
{
	spr_set(     Player_2_SpriteNo );	
	spr_ctrl(	 SIZE_MAS, SZ_16x32);
	spr_x(       Paddle_2_left );
	spr_y(       Paddle_2_top );
    spr_pattern( PaddleAddress );
	spr_pal(     PaddlePaletteNo + 16	);          /* uses sprite palette area */
	spr_pri( 1 );
}
/***************************************************************************/
/* p2_SetupScore() - functions for initializing player 2 score sprites.    */
/*-------------------------------------------------------------------------*/

p2_SetupScore()
{
	spr_set(     Player_2_ScoreLow );
	spr_x(    232  );
	spr_y(      0  );
    spr_pattern( SpritesAddress );
	spr_pal( PaddlePaletteNo + 16	);          /* uses sprite palette area */
	spr_pri( 1 );

	spr_set(     Player_2_ScoreHigh );
	spr_x(    216  );
	spr_y(     0  );
    spr_pattern( SpritesAddress );
	spr_pal(  PaddlePaletteNo + 16	);          /* uses sprite palette area */
	spr_pri( 1 );
}
/***************************************************************************/
/* p2_Setup() - initialize player 2, including his sprite.                 */
/*-------------------------------------------------------------------------*/

p2_Setup()
{
	Paddle_2_Fx    = 0;
    Paddle_2_X     = 240 - 8;
	
	Paddle_2_Fy    = 0;
	Paddle_2_Y     = 120;
	
	Paddle_2_Fdx     = 0;
	Paddle_2_Dx      = 0;
	
	Paddle_2_Fdy     = 0;
	Paddle_2_Dy      = 0;
	
	p2_SetupSprite();

	p2_Score  = 0;
	p2_SetupScore();
	
	CompMoveCount = 1;
	CompMoveDirection = 0;
}

/***************************************************************************/
/* p2_Display() - display hero, including his sprite.                      */
/*-------------------------------------------------------------------------*/

p2_Display()
{
	spr_set(     Player_2_SpriteNo );
	spr_x(       Paddle_2_left );
	spr_y(       Paddle_2_top  );
	spr_show(    Player_2_SpriteNo     );
}
/***************************************************************************/
/* p2_DisplayScore() - display hero, including his sprite.                      */
/*-------------------------------------------------------------------------*/

p2_DisplayScore()
{
	spr_show(    Player_2_ScoreLow   );
	spr_show(    Player_2_ScoreHigh  );
}
/***************************************************************************/
/* p2_ProcessKeys() - figure out what to do with controller inputs.        */
/*-------------------------------------------------------------------------*/

p2_ProcessKeys()
{
    Paddle_2_Fdx = 0;
	Paddle_2_Dx  = 0;
	
	Paddle_2_Fdy = 0;
    Paddle_2_Dy  = 0;
	
	/*.............................................................*/
	/* if player 2 paddle is same as player 1 paddle, he's playing */
	/* the computer. Note that the computer bases it's moves on    */
	/* the centers of the ball and paddle. This means it will avoid*/
	/* the more extreme angles, hopefully giving the player a      */
	/* chance to actually score.                                   */
	/*.............................................................*/
	/* what we should do is use a random counter for how long to   */
	/* move in a given direction.                                  */
	/*.............................................................*/
	
	if( p2_Paddle == 0 )
	{
	   CompMoveCount --;
	   if( CompMoveCount <= 0 )
	   {
		  Rush              = random( RUSH_SEED );
	      CompMoveCount     = random( MOVE_SEED );
		  CompMoveDirection = 0;
		  
	      if( Ball_Y + 3 < Paddle_2_top + 16 )  CompMoveDirection = -1;
	      if( Ball_Y + 3 > Paddle_2_top + 16 )  CompMoveDirection =  1;
	   }
	   
	   if( CompMoveDirection < 0 )  p2_Set_Up_Dy();
	   if( CompMoveDirection > 0 )  p2_Set_Down_Dy();
	   
	   if( Rush <= 1             )        p2_To_Net();
	   if( Rush >= RUSH_SEED - 3 )        p2_From_Net();
	}
	/*.............................................................*/
	/* if player 2 paddle NOT same as player 1 paddle, he's playing*/
	/* a human. Make moves based on paddle input.                  */
	/*.............................................................*/

	else
	{
		if( joyUpDown(   p2_Paddle) ) p2_Set_Up_Dy();
		if( joyDownDown( p2_Paddle) ) p2_Set_Down_Dy();
		if( joyRightDown(p2_Paddle) ) p2_From_Net();
		if( joyLeftDown( p2_Paddle) ) p2_To_Net();
	}
}

/***************************************************************************/
/* p2_Move() - update paddle information.                                  */
/*-------------------------------------------------------------------------*/

p2_Move()
{
	/*.............................................................*/
	/* update paddle position.                                     */
	/*.............................................................*/
	
    p2_Update_X();
	p2_Update_Y();

	/*.............................................................*/
	/* lock paddle into playable area.                             */
	/*.............................................................*/
	
	if( Paddle_2_top < 24 )
	{
	   Paddle_2_Y   = 24;
	   Paddle_2_Dy  = 0;
	}
	if( Paddle_2_bot >= 216 )
	{
	   Paddle_2_Y   = 184;
	   Paddle_2_Dy  = 0;
	}
	if( Paddle_2_left < 137 )
	{
	   Paddle_2_X = 137 ;
	}
	if( Paddle_2_right > 248 )
	{
	   Paddle_2_X = 256 - 16;
	}
}
/***************************************************************************/
/* p2_AddPoint() - Give player a point.		                               */
/*-------------------------------------------------------------------------*/

p2_AddPoint()
{
	int low;
	int high;
	
	/*............................................................*/
	/* you only get points if you are serving.                    */
	/*............................................................*/
	
    if( Serving == 2 )
	{
		p2_Score = p2_Score + 1;
	}

	/*............................................................*/
	/* get digits for sprites.                                    */
	/*............................................................*/
	
	high = p2_Score / 10;
	low  = p2_Score % 10;

	/*............................................................*/
	/* update sprites                                             */
	/*............................................................*/
	
	spr_set(     Player_2_ScoreLow );
    spr_pattern( SpritesAddress + low * 64 );
	
	spr_set(     Player_2_ScoreHigh );
    spr_pattern( SpritesAddress + high * 64 );

	/*............................................................*/
	/* reset paddle motions.                                      */
	/*............................................................*/
	
	paddleReset();

	/*............................................................*/
	/* check for game over.                                       */
	/*............................................................*/
	
	if( p2_Score >= 11 )
	{
	   /*.....................................................*/
	   /* you have to win by 2.                               */
	   /*.....................................................*/
	   
	   if( p2_Score >= (p1_Score + 2) )
	   {
	        winner = 2;
			put_string( "PLAYER 2 WINS!", 8, 0 );
			EndGame();
			return;
	   }

	   /*.....................................................*/
	   /* and 21-21 is a tie.                                 */ 
	   /*.....................................................*/
	   
	   if( p2_Score >= 21 )
	   {
	       if( p1_Score >= 21 )
		   {
				winner = 0;
				put_string( "   TIE GAME   ", 8, 0 );
				EndGame();
				return;
		   }
	   }
	}
}
