/*.....................................................................................*/
/* Massteroids - A Multi-player asteroids game. Hopefully, with realistic physics.     */
/*.....................................................................................*/
/* Copyright 2010 by Michael E. Ward. All Rights Reserved.                             */
/*.....................................................................................*/

#include "huc.h"

/*......................................................................................*/
/* the (more or less) standard includes.                                                */
/*......................................................................................*/

#include "Font/Font.c"
#include "Lib/JoyPad.c"
#include "Lib/tap.c"

#include "Sound/psglib.c"
#include "Sound/sndDefs.h"

/*......................................................................................*/

#define  SCREEN_W        256
#define  SCREEN_H        224

#asm
_SCREEN_W = 256
_SCREEN_H = 224
#endasm

#define  SHOT_SOUND         1
#define  ROCK_SOUND         2
#define  DEAD_SOUND         3
#define  MENU_SONG          4
#define  GAME_SONG          5

#asm
_SHOT_SOUND_ = 1
_ROCK_SOUND_ = 2
_DEAD_SOUND_ = 3
#endasm

/*......................................................................................*/
/* global number of players in the game (NOT maximum #. Actual number playing.)         */
/*......................................................................................*/

int  ShotsPerPlayer;
int  ActivePlayers;

/*......................................................................................*/

#include "table.c"
#include "table2.c"
#include "sqrtab.c"

/*......................................................................................*/
/* game resources.                                                                      */
/*......................................................................................*/

#include "Rocks/Rock.res"
#include "Ships/Ship.res"

#include "Rocks/RockType.h"

/*......................................................................................*/
/* shot information.                                                                    */
/*......................................................................................*/

#include "Shots/Shot.h"
#include "Shots/Shot.c"

/*......................................................................................*/
/* ship information.                                                                    */
/*......................................................................................*/

#include "Ships/Ship.h"
#include "Ships/Ship.c"
#include "Ships/ShipKeys.c"

#include "score/score.c"

/*......................................................................................*/
/* rock information.                                                                    */
/*......................................................................................*/

#include "Rocks/Rock.h"
#include "Rocks/Rock.c"
#include "Rocks/Split.c"
#include "Rocks/Spawn.c"

#include "Menus/TitleScr.c"

#include "Collisions.c"

/*......................................................................................*/
/* the music / effects functions.                                                       */
/*......................................................................................*/

#include "snginit.c"

int  Dead;
int  lvl;
int  tap;

main()
{
    int i;
	
    init_satb();

    /*.........................................................................*/
	/* load game objects.                                                      */
    /*.........................................................................*/

    rockLoad();
    shipLoad();

    /*.........................................................................*/
    /* set up psg / mml player.                                                */
    /*.........................................................................*/

    psgOn( IRQ_TIMER ); 			/* use timer for now : apparently, me and vsync don't get along */
    psgInit( BOTH_300HZ );
  
    /*.........................................................................*/
    /* initialize the song. sngInit() is a squirrel-generated function, that */
    /* handles all of the setup for a song, including registering everything.*/
    /* It is worth looking at to see how things are done to actually get a   */
    /* song ready for playing.                                               */
    /*.........................................................................*/
    
    sngInit();
	psgDelay( 0 );
	
    while( 1 )
    {
		clearScore();
  		
		/*.........................................................................*/
		/* initialize game objects for game.                                       */
		/*.........................................................................*/
		
		shipSetup();
		shotSetup();
		rockSetup();
	  
		/*.........................................................................*/
		/* run the main title.                                                     */
		/*.........................................................................*/

		cls();
		set_font_pal(  FontPaletteNo   );        /* which palette to use for fonts */
		runMainTitle();
		
		/*.........................................................................*/
		/* check for tap.                                                          */
		/*.........................................................................*/
        /* due to a wierd bug I haven't found, we only allow multi-player.         */
		
		/* if( haveTap() != 0 )
		{
		*/
		ActivePlayers = MAX_SHIPS;
 	    runMultiPlayerMenu();
		/*
		}
		else
		{
		   ActivePlayers = 1;
		   runOnePlayerMenu();
		   
		   /*.............................................................*/
		   /* deactivate any lingering ships.                             */
		   /*.............................................................*/
		/*
		   for( i = 1; i < MAX_SHIPS; i ++ )
		   {
		       ShipState[ i ] = SHIP_INACTIVE;
		   }
		   ShipState[ 0 ] = (SHIP_ALIVE | SHIP_ACTIVE);
		}
		*/
	    ShotsPerPlayer = 16 / humans;

		/*.........................................................................*/
		/* activate player ships, and spawn initial rocks.                         */
		/*.........................................................................*/
	  
		shipActivate();
		rockSpawn(  0, 64 );
		
		lvl = 0;
		cls();
		
  	    /*.........................................................................*/
	    /* main game play loop.                                                    */
	    /*.........................................................................*/
	  
	    /* psgPlay( 1 ); 		/* ding */
		psgPlay( GAME_SONG );
		
	    while( ! isGameOver() )
	    {
		  vsync(0);
		  /*
		  set_color( 256, 0000 );
		  UpdateFrameCounter();
		  put_number( FrameRate(), 5, 26, 0 );
	      */
		  
		  showScore();
		  
		  /*...................................................................*/
		  /* handle ship movements.                                            */
		  /*...................................................................*/
	  
		  shipProcessMoves();
		  
		  /*...................................................................*/
		  /* update object positions.                                          */
		  /*...................................................................*/
		  
		  UpdateRocks();        /* update rock positions. Opt Start */
		  UpdateShots();

	      /*
		  put_number( rocksActive, 4, 10, 0 );
		  put_number( shotCount,   4, 15, 0 );
          */
		  /*...................................................................*/
		  /* display objects at new positions.                                 */
		  /*...................................................................*/
		  
		  shipDisplay();
		  DisplayRocks();
		  DisplayShots();

		  /*...................................................................*/
		  /* check for collisions.                                             */
		  /*...................................................................*/
		  
		  checkRockShotCollisions();
		  checkRockShipCollisions();
          checkShotShipCollisions();
		  
		  satb_update();
		  	  
		  if( psgMStat() == 0 )
		  {
			 psgPlay( GAME_SONG );
		  }
		  /*...................................................................*/
		  /* if all rocks are gone, re-spawn.                                  */
		  /*...................................................................*/
		  
		  if( rocksActive == 0 )
		  {
			rockSpawn(  0, 64 );
			lvl++;
			
			if( lvl >= 1 )
				rockSpawn(  8, 32 );
			if( lvl >= 2 )
				rockSpawn( 12, 16 );
			if( lvl >= 3 )
				rockSpawn( 14,  8 );
		  }
		  /*
		  set_color( 256, 0777 );
		  */
	    }
	}
}
/*......................................................................................*/
/* isGameOver() - determine if game is over.                                            */
/*......................................................................................*/

isGameOver()
{
   
   for( ship_no = 0; ship_no < MAX_SHIPS; ship_no++ )
   {   
      /* only check active ships */
	  if( ShipState[ ship_no ] & SHIP_ACTIVE )
	  {
	      /* if it has ships left, game continues */
		  if( ShipLeft[ ship_no ] ) return( 0 );

	      /* if it isn't dead, game continues */
		  if( !(ShipState[ ship_no ] & SHIP_DEAD) ) return( 0 );
	  }
   }
   return( 1 );
}
/*......................................................................................*/
/* reboot() - reboot the system.                                                        */
/*......................................................................................*/

reboot()
{
	#asm
	jmp		_reset
	#endasm
}
