/***************************************************************************/
/* ShipKeys.c - functions for processing ship controls.                    */
/*-------------------------------------------------------------------------*/

char   iShot;			/* temp for shot noises */

/*-------------------------------------------------------------------------*/
/* shipProcessKeys() - dispatch function for ship controls.                */
/*-------------------------------------------------------------------------*/

shipProcessKeys()
{
   if( joyLeftPressed( ship_no  )   )   shipRotateCW();
   if( joyRightPressed( ship_no )   )   shipRotateCCW();
   
   if( joy_II_Down( ship_no )   )   shipThrustOn();
   else                             shipThrustOff();
   
   if( joy_I_Pressed( ship_no ) )   
   {
      if( ShotsFired[ ship_no ] < ShotsPerPlayer )
	  {
		shipFire(); 
		ShotsFired[ ship_no ] ++;
		psgPlay( SHOT_SOUND );
	  }
   }
 
}
/*-------------------------------------------------------------------------*/
/* shipRotateCCW() - rotate ship counter-clockwise.                        */
/*-------------------------------------------------------------------------*/

shipRotateCCW()
{
   #asm
   ldx			_ship_no
   lda			_ShipHeading,x
   dec			a
   and			#$07
   sta			_ShipHeading,x
   #endasm
}
/*-------------------------------------------------------------------------*/
/* shipRotateCW() - rotate ship clockwise.                                 */
/*-------------------------------------------------------------------------*/

shipRotateCW()
{
   #asm
   ldx			_ship_no
   lda			_ShipHeading,x
   inc			a
   and			#$07
   sta			_ShipHeading,x
   #endasm
}
/*-------------------------------------------------------------------------*/
/* shipThrustOn() - activate ship trusters.                                */
/*-------------------------------------------------------------------------*/

shipThrustOn()
{
  #asm
  ldx		_ship_no
  inc		_ShipThrusting,x
  #endasm
}
/*-------------------------------------------------------------------------*/
/* shipThrustOff() - de-activate ship trusters.                            */
/*-------------------------------------------------------------------------*/

shipThrustOff()
{
   #asm
   ldx		_ship_no
   stz		_ShipThrusting,x
   #endasm
}

/*-------------------------------------------------------------------------*/
/* shipFire() - create ship shot sprite, if one is available.              */
/*-------------------------------------------------------------------------*/

shipFire()
{
	#asm

    ;/*.................................................................*/
    ;/* start at next available shot.                                   */
	;/*.................................................................*/

	stz		_iShot					; clear temp shot flag
	ldx		_shot_avail
	
	;/*......................................................................*/
    ;/* if the shot is really available, use it.                             */
    ;/*......................................................................*/
	   
fireLoop
	   lda		_ShotActive,x
	   beq		.cont
	   jmp		.next_shot

    ;	  /*.......................................................*/
	;	  /* shot moves away from ship in current heading direction*/
	;	  /*.......................................................*/
.cont
	      lda		_ship_no
		  sta		_ShotOwner,x
		  tay
		  
		  lda		_ShipHeading,y
		  sta		_ShotHeading,x
          tay							; heading is in Y
		  
		  lda		_cosf,y
		  sta		_ShotDeltaXf, x
		  lda		_cosl,y
		  sta		_ShotDeltaXl, x
		  lda		_cosh,y
		  sta		_ShotDeltaXh, x
		  
		  lda		_sinf,y
		  sta		_ShotDeltaYf, x
		  lda		_sinl,y
		  sta		_ShotDeltaYl, x
		  lda		_sinh,y
		  sta		_ShotDeltaYh, x
		  
	;	  /*.......................................................*/
	;	  /* position shot at end of ship.                         */
	;	  /*.......................................................*/
		  
		  tya
		  asl	a				; we need 2 byte offset (heading) for sprite offsets
		  sta	_shot_heading_offset
		  
		  clc
		  stz	_ShotScreenXf,x
		  ldy   _ship_no
		  lda	_ShipScreenXl,y
		  ldy	_shot_heading_offset
		  adc	_ShotOffsetX,y
		  sta	_ShotScreenXl,x
		  
		  ldy   _ship_no
		  lda	_ShipScreenXh,y
		  ldy	_shot_heading_offset
		  adc	_ShotOffsetX+1,y
		  sta	_ShotScreenXh,x

		  clc
		  stz	_ShotScreenYf,x
		  ldy   _ship_no
		  lda	_ShipScreenYl,y
		  ldy	_shot_heading_offset
		  adc	_ShotOffsetY,y
		  sta	_ShotScreenYl,x
		  
		  ldy   _ship_no
		  lda	_ShipScreenYh,y
		  ldy	_ShotHeading,x
		  ldy	_shot_heading_offset
		  sta	_ShotScreenYh,x

	;	  /*.......................................................*/
	;	  /* ship recoils in opppsite direction from shot.         */
	;	  /*.......................................................*/
	;	  /* Recoil doesn't work right now.                        */

	;	  shipRecoil();

	call		_shipRecoil
		  
  	;      ShipDeltaX = ShipDeltaX - ( cos[ ShipHeading * 45 ] /16 );
	;      ShipDeltaY = ShipDeltaY - (-sin[ ShipHeading * 45 ] /16 );
    ;      */

			inc		_ShotActive,x		; activate this shot
			inc		_iShot				; mark as having shot
		  
	;	  /*.......................................................*/
	;	  /* move to next shot, and wrap if needed.                */
	;	  /*.......................................................*/

		  txa
		  inc	a
		  cmp	#_MAX_SHOTS
		  bne	.isOkay
		  cla
.isOkay
          tax
          sta		_shot_avail		; update next available shot
          rts		  				; and we're done

    ;------------------------------------------------------------
	; shot was already active. try next
	
.next_shot
       inx
	   cpx		#_MAX_SHOTS
	   bcc		.no_wrap
	   clx						; wrap shot counter
.no_wrap
       cpx		_shot_avail		; went all the way around
	   beq		exit1
	   jmp		fireLoop
exit1
	   rts
       #endasm
}