/***************************************************************************/
/* Shot.c - functions for managing shots.                                  */
/*-------------------------------------------------------------------------*/

int  shotCount;

int  which;
int  whom;

char  shot_heading_offset;

/***************************************************************************/
/* shotSetup() - initialize shots, including their sprites.                */
/*-------------------------------------------------------------------------*/

shotSetup()
{
    int  i;
	
    /*............................................................*/
	/* mark all shots as available.                               */
	/*............................................................*/
	
	for( i = 0; i < MAX_SHOTS; i ++ )
	{
	   ShotActive[ i ]         = 0;
	   ShotCurrentAddress[ i ] = RockAddress + 0x580;
	   
	   spr_set(     FIRST_SHOT + i );
	   spr_pattern( ShotCurrentAddress[ i ] );
  	   spr_pal(     RockPaletteNo + 16	    );
	   spr_hide(    FIRST_SHOT + i );
	   spr_pri( 1 );
	}
	/*............................................................*/
	/* next available one is 0                                    */
	/*............................................................*/
	
	shot_avail = 0;
}
/***************************************************************************/
/* DisplayShots() - display ALL shot sprites.                              */
/*-------------------------------------------------------------------------*/
/* uses X internally, makes no effort to preserve registers.               */

DisplayShots()
{
   #asm

   clx							; start at shot 0
shot_d_loop

   ;-------------------------------------------------------------------------
   ; loop through shots.
   
   phx							; save shot number
   lda		_ShotActive,x		; is it active ?
   beq		.skipit
   
	;...................................................................
	; spr_set

	txa						; get shot into accumlator
	clc						; convert to sprite number
	adc		#_FIRST_SHOT

	pha						; save sprite id
	phx						; and shot no

	tax						; sprite number goes in X. we can ignore A
	call  	_spr_set  		; call HuC function. has to be call, to make
							; sure it's mapped in.

	;...................................................................
	; spr_x

	plx						; peek at shot no
    phx
		
	sec							; adust coordinate
	lda		_ShotScreenXl,x
	sbc		#LOW( 3 )
	pha							; save for later
	lda		_ShotScreenXh,x
	sbc		#HIGH( 3 )

	plx							; restore low byte
	call		_spr_x  
			
	;...................................................................
	; subtract low byte

	plx							; restore shot no
	
	sec
	lda		_ShotScreenYl,x		; adust coordinate
	sbc		#LOW( 3 )
	pha							; save for later
	lda		_ShotScreenYh,x
	sbc		#HIGH( 3 )
	plx							; restore low byte
	call		_spr_y
			
	;...................................................................
	; spr_show
    ;...................................................................
	; spr_show and spr_hide use Y to indicate which sprite # to use.
	; y == 0 => already set sprite; y != 0 => giving different sprite
	; this can cause major headaches, when sprites don't show/hide :-)
	
    pla							; restore sprite id
	cly							; use already set sprite
	call  	_spr_show
			
   ;-------------------------------------------------------------------------
   ; shot done; move to next shot
   
.skipit
   plx						; restore shot number
   inx						; next shot
   cpx		#_MAX_SHOTS		; all done ?
   bcc		shot_d_loop		; nope, keep going
   
   #endasm
}
/***************************************************************************/
/* UpdateShots() - update shot sprites.                                    */
/*-------------------------------------------------------------------------*/

UpdateShots()
{
   #asm

   stz		_shotCount	
   clx						; shot number is in X through whole routine
   
shot_u_loop

   ;---------------------------------------------------------------------
   ; if shot is active...
   
   lda		_ShotActive,x
   
   ;.....................................................................
   ; this may be needed if code gets added
   ;   bne		.gotShot
   ;   jmp      noShot			; shot not active, do next
   
   beq		noShot

   ;---------------------------------------------------------------------
   ; .... update shot position.
   
.gotShot

   jsr		_shotAddDeltas		; this uses X, but doesn't trash it
   
  ;---------------------------------------------------------------------
  ; shot is active, did it go off-screen ?
  ;-------------------------------------------------------------------------------------
  ; if X < 0, shot no longer active.
  
  lda		_ShotScreenXh,x
  cmp		#$ff				; did coordinate go negative ?
  beq		.clrActive

  ;-------------------------------------------------------------------------------------
  ; if X >= 256, shot no longer active.
  
  lda		_ShotScreenXh,x
  cmp		#$01				; did coordinate go off bottom?
  beq		.clrActive

  ;-------------------------------------------------------------------------------------
  ; if Y < 0, shot no longer active.

  lda		_ShotScreenYh,x
  cmp		#$ff
  beq		.clrActive
  
  ;-------------------------------------------------------------------------------------
  ; if Y >= 224, shot no longer active.

  lda		_ShotScreenYl,x
  cmp		#224
  bcs		.clrActive
  
  ;--------------------------------------------------------------------------
  ; if none of the above, shot is still active
  
  inc		_shotCount
  jmp		noShot

	;--------------------------------------------------------------------------
	; mark shot as inactive.
	
.clrActive  

  stz		_ShotActive, x
  lda		_ShotOwner,x			; get owning ship
  phx
  tax
  dec		_ShotsFired,x			; free shot
  plx

  
  ;--------------------------------------------------------------------------
  ;  spr_hide( FIRST_SHOT + shot_no  );
  ;...........................................................................
  ; there are two ways to call spr_hide. y == 0 => use already set sprite
  ;                                      y != 0 => you're giving sprite #
  ; be careful with this. it's easy to screw up.....
	
  phx						; save index

  txa						; load shot #
  clc
  adc	    #_FIRST_SHOT
  tax						; sprite number goes in X. we can ignore A
  ldy	#1					; ...but Y must be non-0 to hide sprite by id
  call  _spr_hide  			; call HuC function. has to be call, to make
                            ; sure it's mapped in.
  plx
  
  ;--------------------------------------------------------------------------
  ; shot still on screen

noShot

  ;--------------------------------------------------------------------------
  ; handle next shot
  
  inx
  cpx		#_MAX_SHOTS
  bcs		.shotOut
  jmp		shot_u_loop
  
.shotOut  
  #endasm
}
/***************************************************************************/
/* shotAddDeltas() - add DeltaX and DeltaY to shot coordinates.            */
/*-------------------------------------------------------------------------*/
/* this is a 3 byte add, with fractions in the lsb.                        */
/*.........................................................................*/

shotAddDeltas()
{
#asm
_shotAddDeltas

	;-----------------------------------------------------------------
	; update ScreenX
	
	clc
	lda		_ShotDeltaXf,x
	adc		_ShotScreenXf,x
	sta		_ShotScreenXf,x
	
	lda		_ShotDeltaXl,x
	adc		_ShotScreenXl,x
	sta		_ShotScreenXl,x
	
	lda		_ShotDeltaXh,x
	adc		_ShotScreenXh,x
	sta		_ShotScreenXh,x
	
	;-----------------------------------------------------------------
	; update ScreenY
	
	clc
	lda		_ShotDeltaYf,x
	adc		_ShotScreenYf,x
	sta		_ShotScreenYf,x
	
	lda		_ShotDeltaYl,x
	adc		_ShotScreenYl,x
	sta		_ShotScreenYl,x
	
	lda		_ShotDeltaYh,x			; check sign bit
	adc		_ShotScreenYh,x
	sta		_ShotScreenYh,x
	
#endasm
}
